﻿using System;
using System.Diagnostics;
using System.Drawing;
using System.Windows.Forms;

namespace TreeLabelEdit {
    internal partial class MainForm : Form {
        public MainForm() {
            InitializeComponent();
            tree.TreeViewNodeSorter = new NodeSorter();
        }

        private void Form_Load(object sender, EventArgs e) {
            tree.Nodes.Add(new TreeNode("A", 0, 0,
                new TreeNode[] { 
                    new TreeNode("A1", 1, 1),
                    new TreeNode("A2", 1, 1),
                    new TreeNode("A3", 1, 1),
                    new TreeNode("AX", 0, 0,
                        new TreeNode[] { 
                            new TreeNode("AX1", 1, 1),
                            new TreeNode("AX2", 1, 1),
                            new TreeNode("AX3", 1, 1),
                        })
                }));

            tree.Nodes.Add(new TreeNode("B", 0, 0,
                new TreeNode[] { 
                    new TreeNode("B1", 1, 1),
                    new TreeNode("B2", 1, 1),
                    new TreeNode("B3", 1, 1) 
                }));

            tree.Nodes.Add(new TreeNode("C", 0, 0,
                new TreeNode[] { 
                    new TreeNode("C1", 1, 1),
                    new TreeNode("C2", 1, 1),
                    new TreeNode("C3", 1, 1) 
                }));

            tree.Nodes.AddRange(new TreeNode[] { 
                new TreeNode("1", 1, 1),
                new TreeNode("2", 1, 1),
                new TreeNode("3", 1, 1) 
            });

            tree.ExpandAll();
            tree.Sort();
        }

        private void tree_MouseDown(object sender, MouseEventArgs e) {
            var node = tree.GetNodeAt(e.Location);
            if ((node != null) && !node.Equals(tree.SelectedNode)) { //to allow proper selecting
                tree.SelectedNode = node;
            }
        }


        private void tree_KeyDown(object sender, KeyEventArgs e) {
            if (e.KeyData == Keys.F2) {
                if (tree.SelectedNode != null) { tree.SelectedNode.BeginEdit(); }
            }
        }

        private void tree_BeforeLabelEdit(object sender, NodeLabelEditEventArgs e) {
            e.CancelEdit = (e.Node.ImageIndex == 0); //don't allow editing folders
        }

        private void tree_AfterLabelEdit(object sender, NodeLabelEditEventArgs e) {
            if (e.Label == null) { return; } //no change was made

            e.CancelEdit = true; //we will handle changes manually

            string newText = e.Label.Trim(); //no spaces

            var nodes = (e.Node.Parent == null) ? tree.Nodes : e.Node.Parent.Nodes;
            foreach (TreeNode node in nodes) {
                if ((node != e.Node) && string.Equals(newText, node.Text, StringComparison.Ordinal)) {
                    return; //duplicate name
                }
            }

            e.Node.Text = newText; //rename manually

            tree.BeginInvoke(new Action<TreeNode>(delegate(TreeNode node) { //sort again
                tree.BeginUpdate();
                tree.Sort();
                tree.SelectedNode = node;
                tree.EndUpdate();
            }), e.Node);
        }

    }
}
